unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Controls, Forms,
  Dialogs, Contnrs, Buttons, iDragDrop, Menus, ExtCtrls, StdCtrls;

type
  TMainForm = class(TForm)
    OpenFile: TOpenDialog;
    PopupMenu: TPopupMenu;
    PMenuShowInfo: TMenuItem;
    PMenuCheckOpen: TMenuItem;
    PMenuSetPassword: TMenuItem;
    PMenuDelFromList: TMenuItem;
    PnlButtons: TPanel;
    BtnAddFile: TBitBtn;
    ChkAddFile: TCheckBox;
    EdtExts: TEdit;
    BtnCheck: TBitBtn;
    BtnLetGo: TBitBtn;
    Panel1: TPanel;
    LstFiles: TListBox;
    Splitter1: TSplitter;
    MemResult: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BtnAddFileClick(Sender: TObject);
    procedure LstFilesDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure BtnCheckClick(Sender: TObject);
    procedure PMenuShowInfoClick(Sender: TObject);
    procedure LstFilesMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BtnLetGoClick(Sender: TObject);
  private
    Dropper: TShellDrop;
    WorkSL: TStringList;
    procedure AddFiles(Files: TStrings);
    procedure DroppedAction(AFiles, AFolds: TStringList;
            AControl: TControl; APos: TPoint);
    procedure DoMessage(const Msg: string; const Level: Byte = 0);
  public

  end;

var
  MainForm: TMainForm;

implementation

uses
    StrUtils, ShellApi, iado, iadodata, ScanFile;

{$R *.dfm}


//  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  _CAO
function SimpleMessage(const Msg: string; const DlgType: TMsgDlgType; const Buttons: TMsgDlgButtons): integer;
var
    Dialog: TForm;
    Image : TImage;
begin
    Application.NormalizeTopMosts;

    Dialog := CreateMessageDialog(Trim(Msg), DlgType, Buttons);
    try
        Dialog.Caption := Application.Title + ' - ' + Dialog.Caption; 
        Image := TImage(Dialog.FindComponent('Image'));
        if (Image <> nil) then Image.Picture.Icon.Assign(Application.Icon);
        Result := Dialog.ShowModal;
    finally
        Dialog.Free;
    end;

    Application.RestoreTopMosts;
end;

//  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  
function SimpleQuery(const Prompt: string; var Value: string): Boolean;
begin
    Result := InputQuery(Application.Title, Prompt, Value);
end;

//  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  ShellExecute
procedure SimpleExecute(const Path: string);
begin
    ShellExecute(Application.Handle, nil, PChar(Path), nil, nil, SW_SHOW);
end;

//  ---------------------------------------------------------------------------
//  RXgN^
procedure TMainForm.FormCreate(Sender: TObject);
begin
    Self.Caption := Application.Title + ' - ' + Self.Caption; 
    WorkSL := TStringList.Create;
    Dropper := TShellDrop.Create(LstFiles, DroppedAction);
    Dropper.Start();
end;

//  ---------------------------------------------------------------------------
//  fXgN^
procedure TMainForm.FormDestroy(Sender: TObject);
begin
    Dropper.Free;
    WorkSL.Free;
end;

//  ---------------------------------------------------------------------------
//  t@Cǉ()
procedure TMainForm.DoMessage(const Msg: string; const Level: Byte = 0);
const
    S_LEVEL = '  ';
var
    T: string;
begin
    T := FormatDateTime('hh:nn:ss', Now());
    MemResult.Lines.Add(Format('%s %s%s', [T, DupeString(S_LEVEL, Level), Msg]));
end;

//  ---------------------------------------------------------------------------
//  t@Cǉ()
procedure TMainForm.AddFiles(Files: TStrings);
var
    MdbInfo: TMdbFile;
    Path, Path2: string;
    i, index: integer;
    SL: TStringList;
begin
    //  t@Ct@Ĉݗ
    SL := TStringList.Create;
    SL.Sorted := true;
    for i := 0 to MdbFiles.Count - 1 do
    begin
        MdbInfo := TMdbFile(MdbFiles[i]);
        SL.Add(MdbInfo.DispName);
    end;

    //  ɑ݂Ăꍇ͖(t@C͕s)
    for i := 0 to Files.Count - 1 do
    begin
        Path  := Files[i];
        Path2 := ExtractFileName(Path);
        if not SL.Find(Path2, index) then
        begin
            MdbFiles.Add(TMdbFile.Create(Path));
            SL.Add(Path2);
        end;
    end;

    SL.Free;

    //  sݒ
    LstFiles.Count := MdbFiles.Count;
end;

//  ---------------------------------------------------------------------------
//  t@Cǉ(_CAOoR)
procedure TMainForm.BtnAddFileClick(Sender: TObject);
begin
    if OpenFile.Execute then
    begin
        if not ChkAddFile.Checked then
        begin
            MdbFiles.Clear;
            LstFiles.Count := 0;
        end;
        AddFiles(OpenFile.Files);
    end;
end;

//  ---------------------------------------------------------------------------
//  t@Cǉ(hbOAhhbvoR)
procedure   TMainForm.DroppedAction(AFiles, AFolds: TStringList;
            AControl: TControl; APos: TPoint);
var
    Res: TStrings;
begin
    if not ChkAddFile.Checked then
    begin
        MdbFiles.Clear;
        LstFiles.Count := 0;
    end;
    Res := TStringList.Create();
    try
        Dropper.Stop();
        ScanFiles(EdtExts.Text, AFiles, AFolds, nil, Res);
        AddFiles(Res);
    finally
        Dropper.Start();
        Res.Free;
    end;
end;

//  ---------------------------------------------------------------------------
//  \̓t@Ĉ
procedure TMainForm.LstFilesDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
    LB: TListBox;
    MdbInfo: TMdbFile;
    OffsetX, OffsetY: integer;
    Item: string;
begin
    //  LXg
    LB := TListBox(Control);
    //  t@CNXo
    MdbInfo := TMdbFile(MdbFiles[Index]);
    //  \o
    Item :=  Format('%s t@C[%s] pX[h[%s]', [S_STATUS1[MdbInfo.Status], MdbInfo.DispName, MdbInfo.Password]);
    //  \ʒu
    OffsetX := 5;
    Inc(OffsetX, Rect.Left);
    OffsetY := ((Rect.Bottom - Rect.Top) - LB.Canvas.TextHeight(Item)) div 2;
    Inc(OffsetY, Rect.Top);
    //  `
    LB.Canvas.TextRect(Rect, OffsetX, OffsetY, Item);
end;

//  ---------------------------------------------------------------------------
//  ڑ`FbN(`FbN̂̂݉͑ΏۂƂȂ)
procedure TMainForm.BtnCheckClick(Sender: TObject);
var
    i, Count: integer;
    MdbInfo: TMdbFile;
    AdoJet: TAdoxJetBase;
begin
    Screen.Cursor := crHourGlass;
    PnlButtons.Enabled := false;
    AdoJet := TAdoxJetBase.Create;
    MemResult.Clear;
    try
        DoMessage('ڑ`FbNJn');
        Count := MdbFiles.Count;
        for i := 0 to Count - 1 do
        begin
            //  t@CNXo
            MdbInfo := TMdbFile(MdbFiles[i]);
            DoMessage(Format('[%2d/%2d] `FbNJn %s', [i + 1, Count, MdbInfo.DispName]));
            //  `FbN̏ꍇ̂
            if (MdbInfo.Status = fsUnknown) then
            begin
                try
                    //  ڑĂ݂
                    AdoJet.OpenConnection(MdbInfo.FileName, MdbInfo.Password);
                    //  ؒf
                    AdoJet.CloseConnection();
                    //  Ȃ
                    MdbInfo.Status := fsNoProblem;
                except
                    //  蔭
                    on E:Exception do
                    begin
                        MdbInfo.Msg := E.Message;
                        if (AnsiPos('pX[h', MdbInfo.Msg) > 0) then
                            MdbInfo.Status := fsNeedPassword
                        else
                            MdbInfo.Status := fsError;
                    end;
                end;
            end;
            DoMessage(Format('%s %s', [S_STATUS1[MdbInfo.Status], MdbInfo.Msg]));
            LstFiles.Repaint();
        end;
        DoMessage('ڑ`FbNI');
    finally
        AdoJet.Free;
        PnlButtons.Enabled := true;
        Screen.Cursor := crDefault;
    end;
end;

//  ---------------------------------------------------------------------------
//  |bvAbvj[\
procedure TMainForm.LstFilesMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
    index: integer;
    Pt: TPoint;
begin
    //  NbNʒuɃACe邩
    index := TListBox(Sender).ItemAtPos(Point(X, Y), true);

    //  ENbN̏ꍇ
    if (index > -1) and (Button = mbRight) then
    begin
        //  I
        TListBox(Sender).ItemIndex := index;
        //  |bvAbv
        Pt := TListBox(Sender).ClientToScreen(Point(X, Y));
        PopupMenu.Popup(Pt.X, Pt.Y);
    end;
end;

//  ---------------------------------------------------------------------------
//  낢ƃACeƂ̂
procedure TMainForm.PMenuShowInfoClick(Sender: TObject);
var
    index: integer;
    MdbInfo: TMdbFile;
    AdoJet: TAdoxJetBase;
    Temp: string;
begin
    //  IĂACe𒲂ׂ
    index := LstFiles.ItemIndex;
    if (index = -1) then exit;
    //  IuWFNgoĂ
    MdbInfo := TMdbFile(MdbFiles[index]);

    //  
    if (Sender = PMenuShowInfo) then
    begin
        WorkSL.Clear;
        WorkSL.Add(Format('t@C'#9'%s', [MdbInfo.DispName]));
        WorkSL.Add(Format('tpX@'#9'%s', [MdbInfo.FileName]));
        WorkSL.Add(Format('Xe[^X'#9'%s', [S_STATUS2[MdbInfo.Status]]));
        WorkSL.Add(Format('bZ[W'#9'%s', [MdbInfo.Msg]));
        WorkSL.Add(Format('pX[h'#9'%s', [MdbInfo.Password]));
        SimpleMessage(WorkSL.Text, mtInformation, [mbOK]);
    end
    else
    //  ڑ`FbN
    if (Sender = PMenuCheckOpen) then
    begin
        AdoJet := TAdoxJetBase.Create;
        try
            //  `FbN̏ꍇ̂
            if (MdbInfo.Status = fsUnknown) then
            begin
                try
                    //  ڑĂ݂
                    AdoJet.OpenConnection(MdbInfo.FileName, MdbInfo.Password);
                    //  ؒf
                    AdoJet.CloseConnection();
                    //  Ȃ
                    MdbInfo.Status := fsNoProblem;
                except
                    //  蔭
                    on E:Exception do
                    begin
                        MdbInfo.Msg := E.Message;
                        if (AnsiPos('pX[h', MdbInfo.Msg) > 0) then
                            MdbInfo.Status := fsNeedPassword
                        else
                            MdbInfo.Status := fsError;
                    end;
                end;
            end;
        finally
            AdoJet.Free;
        end;
    end
    else
    //  pX[hݒ
    if (Sender = PMenuSetPassword) then
    begin
        Temp := MdbInfo.Password;
        if (SimpleQuery('pX[hݒł܂B', Temp)) then
        begin
            MdbInfo.Password := Temp;
        end;
    end
    else
    //  폜
    if (Sender = PMenuDelFromList) then
    begin
        //  폜
        MdbFiles.Delete(index);
        //  sݒ
        LstFiles.Count := MdbFiles.Count;
        //  Iʒu
        if (LstFiles.Count > 0) then
        begin
            if (LstFiles.Count <= index) then Dec(index);
            LstFiles.ItemIndex := index;
        end;
    end;
    //  ĕ`
    LstFiles.Repaint();
end;

//  ---------------------------------------------------------------------------
//  ͊Jn
procedure TMainForm.BtnLetGoClick(Sender: TObject);
var
    Msg, OutPath: string;
    JetHtml: TAdoxJetHtml;
begin
    //  mF
    Msg := 'ڑ`FbNς݂̃t@C([' + S_STATUS1[fsNoProblem] + '] ̃t@C)͂܂B'
         + #13#10'Ԃ\\łB͂n߂܂H';
    if (SimpleMessage(Msg, mtConfirmation, [mbYes, mbNo]) = mrNo) then exit;
    //  O
    Screen.Cursor := crHourGlass;
    PnlButtons.Enabled := false;
    MemResult.Clear;
    //  o̓tH_pX
    OutPath := ExtractFilePath(ParamStr(0)) + FormatDateTime('"mdberput"yyyymmdd', Now()) + '\';
    JetHtml := TAdoxJetHtml.Create();
    try
        JetHtml.DoAnalyze(OutPath, MemResult.Lines);
    finally
        JetHtml.Free;
        PnlButtons.Enabled := true;
        Screen.Cursor := crDefault;
    end;
    //  o̓tH_J
    if DirectoryExists(OutPath) then
    begin
        if (SimpleMessage('o̓tH_J܂H', mtConfirmation, [mbYes, mbNo]) = mrYes) then
        begin
            SimpleExecute(OutPath);
        end;
    end;
end;

end.



